<?php

namespace Modules\ProductCatalogue\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Routing\Controller;
use App\Product;
use App\Contact;
use App\User;
use App\Business;
use App\Discount;
use App\SellingPriceGroup;
use App\Utils\ProductUtil;
use App\BusinessLocation;
use App\Utils\ModuleUtil;
use App\Notifications\PosNotification;
use App\Category;
use Cart;
class ProductCatalogueController extends Controller
{
    /**
     * All Utils instance.
     *
     */
    protected $productUtil;
    protected $moduleUtil;

    /**
     * Constructor
     *
     * @param ProductUtils $product
     * @return void
     */
    public function __construct(ProductUtil $productUtil, ModuleUtil $moduleUtil)
    {
        $this->productUtil = $productUtil;
        $this->moduleUtil = $moduleUtil;
    }

    /**
     * Display a listing of the resource.
     * @return Response
     */
    public function index($business_id, $location_id)
    {
  

            $products = Product::where('business_id', $business_id)
                ->whereHas('product_locations', function($q) use ($location_id){
                    $q->where('product_locations.location_id', $location_id);
                    if(request()->get('category_id') > 0){
                        $category_id=request()->get('category_id');
                        $q->where('products.category_id',$category_id);
                    }
                
                })
                ->ProductForSales()
                ->with(['variations', 'variations.product_variation', 'category'])
              //  ->leftJoin('variation_location_details as vld','vld.product_id','products.id')
                ->get()
                ->groupBy('category_id') ;; 
      
            foreach ($products as $row){
                foreach($row as $pro){
                    $qty=\DB::table('variation_location_details')->where('product_id',$pro->id)->sum('qty_available');
                    if(request()->get('exist_product')){
                        $pro->qty_available=$qty;
                    }else{
                        $pro->qty_available=1;
                    }
                }
            }
        
        // dd( $products );
        $business = Business::with(['currency'])->findOrFail($business_id);
        $business_location = BusinessLocation::where('business_id', $business_id)->findOrFail($location_id);

        $now = \Carbon::now()->toDateTimeString();
        $discounts = Discount::where('business_id', $business_id)
                                ->where('location_id', $location_id)
                                ->where('is_active', 1)
                                ->where('starts_at', '<=', $now)
                                ->where('ends_at', '>=', $now)
                                ->orderBy('priority', 'desc')
                                ->get();

        $categories = Category::forDropdown($business_id, 'product');

             if(request()->get('category_id') > 0){
                        $categories = Category::where('business_id', $business_id)
                                        ->where('id',request()->get('category_id'))
                                        ->where('parent_id', 0)
                                    
                                        ->select(\DB::raw('IF(short_code IS NOT NULL, CONCAT(name, "-", short_code), name) as name'), 'id')
                                        ->orderBy('name', 'asc')
                                        ->get();
            
                        $categories =  $categories->pluck('name', 'id');
            }
        \Session::put('business_id',$business_id);
        \Session::put('location_id',$location_id);
        \Session::put('main_url',\URL::current());
        
        return view('productcatalogue::catalogue.index')->with(compact('products', 'business', 'discounts', 'business_location', 'categories'));
    }
    public function information($business_id,$location_id){
        $business=\DB::table('business')->where('id',$business_id)->first();
        $location=\DB::table('business_locations')->where('id',$location_id)->first();
        if(!$business || !$location){
            return redirect()->back();
        }
        
        return view ('productcatalogue::catalogue.information',compact('location_id','business_id','location','business'));
    }

    /******************************
     * Show the specified resource.
     * @param int $id
     * @return Response
     */
    public function show($business_id, $id)
    {
        $product = Product::with(['brand', 'unit', 'category', 'sub_category', 'product_tax', 'variations', 'variations.product_variation', 'variations.group_prices', 'variations.media', 'product_locations', 'warranty'])->where('business_id', $business_id)
                        ->findOrFail($id);

        $price_groups = SellingPriceGroup::where('business_id', $product->business_id)->active()->pluck('name', 'id');

        $allowed_group_prices = [];
        foreach ($price_groups as $key => $value) {
            $allowed_group_prices[$key] = $value;
        }

        $group_price_details = [];
        $discounts = [];
        foreach ($product->variations as $variation) {
            foreach ($variation->group_prices as $group_price) {
                $group_price_details[$variation->id][$group_price->price_group_id] = $group_price->price_inc_tax;
            }

            $discounts[$variation->id] = $this->productUtil->getProductDiscount($product, $product->business_id, request()->input('location_id'), false, false, $variation->id);
        }

        $combo_variations = [];
        if ($product->type == 'combo') {
            $combo_variations = $this->productUtil->__getComboProductDetails($product['variations'][0]->combo_variations, $product->business_id);
        }

        return view('productcatalogue::catalogue.show')->with(compact(
            'product',
            'allowed_group_prices',
            'group_price_details',
            'combo_variations',
            'discounts'
        ));
    }

    public function generateQr()
    {
        $business_id = request()->session()->get('user.business_id');
        if (!(auth()->user()->can('superadmin') || $this->moduleUtil->hasThePermissionInSubscription($business_id, 'productcatalogue_module'))) {
            abort(403, 'Unauthorized action.');
        }

        $business_id = request()->session()->get('user.business_id');
        $business_locations = BusinessLocation::forDropdown($business_id);
        $business = Business::findOrFail($business_id);

        return view('productcatalogue::catalogue.generate_qr')
                    ->with(compact('business_locations', 'business'));
    }
    public function addToCart(Request $request){
      //  dd($request->variation_id);
      $input=$request->except('_token');
     
       $q = Product::where('id', $request->product_id)
                
                ->ProductForSales()
                ->with(['variations', 'variations.product_variation', 'category']);
                
            $pros= $q->first();
                if($request->variation_id !==null){
                    $var= \DB::table('variations')->where('id',$request->variation_id)->first();;
                }else{
                      $var= \DB::table('variations')->where('product_id',$request->product_id)->first();;
                }
      // dd($var);is_numeric
      $price=$var->default_sell_price;
      if(is_numeric($request->price_group)){
          $price_group=\DB::table('variation_group_prices')->where('variation_id',$var->id)->where('price_group_id',$request->price_group)->first();
          $price=$price_group->price_inc_tax;
      }
        $rows  = Cart::content();
      //   dd($pros);
        //return $rowId = $rows->where('id', $request->product_id)->first()->rowId;
        
        Cart::add($request->product_id, $pros->name, 1, $price,0, ['var_id'=>$var->id],1,0,0);
        
       // $ca=Cart::content()->where('id',$request->product_id)->first()->rowId;
        return Cart::count();
    }
    public function updateCart(){
        return Cart::count();
    }
    
    
    public function myCart(){
          $ca=Cart::content();
           $business_id=\Session::get('business_id');
           $location_id=\Session::get('location_id');
          $services=\DB::table('types_of_services')->where('business_id',$business_id)->get();
          $res_tables=\DB::table('res_tables')->where('location_id',$location_id)->get();
          
        return view('productcatalogue::catalogue.mycart',compact('services','res_tables'));
    }
    public function deleteFromCart(Request $request){
        $rowId=Cart::content()->where('id',$request->product_id)->first()->rowId;
        Cart::remove($rowId);
        
        return 1;
    }
    function updateTable(){
         $ca=Cart::content();
        return view('productcatalogue::catalogue.cart_table');
    }
    
    public function sell(Request $request){
        $lines=$request->product_data;
        if(empty($lines)){
            return redirect()->back()->with('error','عفوا لا يوجد منتجات في السلة ! ');
        }
        if(empty($request->customer_name) || empty($request->customer_phone) ){
            return redirect()->back()->with('error','يرجي كتابة بيانات العميل ');
        }
        \DB::beginTransaction();
       //dd($lines[0]["'product_id'"]);
        $business_id=\Session::get('business_id');
        $location_id=\Session::get('location_id');
        $input_contact['type']='customer';
        $input_contact['name']=$request->customer_name;
        $input_contact['shipping_address']=$request->customer_addres;
        $input_contact['mobile']=$request->customer_phone;
        $input_contact['business_id']=$business_id;
        $admin= User::where('business_id',$business_id)->orderBy('id','asc')->first();
        $input_contact['created_by']=$admin->id;
        $cont=\DB::table('contacts')->where('mobile',$request->customer_phone)->first();
        if($cont==NULL){
            $customer_name=$request->customer_name;
            $contact_id=\DB::table('contacts')->insertGetId($input_contact);

        }else{
             unset($input_contact['name']);
             $customer_name=$cont->name;
            \DB::table('contacts')->where('id',$cont->id)->update($input_contact);
            $contact_id=$cont->id;
        }
     
        $users=User::where('business_id',$business_id)->get();; //\DB::table('contacts')->where('id',$contact_id)->first();
        
        $input_trans['business_id']=$business_id;
        $input_trans['location_id']=$location_id;
        $input_trans['contact_id']=$contact_id;
        $input_trans['created_by']=$admin->id;
        $input_trans['final_total']=$request->inv_total;
        $input_trans['type']='sell';
        $input_trans['status']='draft';
        $input_trans['sub_status']='quotation';
        $input_trans['payment_status']='due';
        $input_trans['transaction_date']=date("Y-m-d h:i:s");
        $input_trans['created_at']=date("Y-m-d h:i:s");
        $input_trans['total_before_tax']=$request->inv_total;
        $input_trans['shipping_address']=$request->customer_addres;

        
        if(!empty($request->res_table_id)){
            $input_trans['res_table_id']=$request->res_table_id;
        }
        
        if(!empty($request->types_of_service_id)){
            $input_trans['types_of_service_id'] = $request->types_of_service_id;
            $services=\DB::table('types_of_services')->where('id',$request->types_of_service_id)->first();
            $input_trans['packing_charge']=$services->packing_charge;
            $input_trans['packing_charge_type']=$services->packing_charge_type;
        }
        
        
        
       $trans_id=\DB::table('transactions')->insertGetId($input_trans);
       
      
     
      foreach($lines as $line){
         
          $detail['transaction_id']=$trans_id;
          $detail['product_id']=$line["'product_id'"];
          $detail['variation_id']=$line["'var_id'"];
          $detail['quantity']=$line["'product_qty'"];
          $detail['unit_price_before_discount']=$line["'unit_price'"];
          $detail['unit_price']=$line["'unit_price'"];
          $detail['unit_price_inc_tax']=$line["'unit_price'"];
          \DB::table('transaction_sell_lines')->insert($detail);
      }
      
      $details=[
          'customer_name'=>$customer_name,
          'transaction_id'=>$trans_id
          ];
          foreach($users as $user){
              \Notification::send($user, new PosNotification($details));
          }
       
      Cart::destroy();
      \DB::commit();
      $output=[
          'success'=>1,
          'msg'=>'تم ارسال الفاتورة بنجاح'
          ];
          $url=\Session::get('main_url');
          return redirect()->back()->with('success',$output['msg']);
    }



public function getProducts()
    {
        if (request()->ajax()) {
            $search_term = request()->input('term', '');
            $location_id = request()->input('location_id', null);
            $hide_pos= request()->input('hide_pos', false);
            
            $check_qty = request()->input('check_qty', false);
            $price_group_id = request()->input('price_group', null);
            $business_id = request()->session()->get('user.business_id') ? request()->session()->get('user.business_id') : request()->input('business_id');
            $not_for_selling = request()->get('not_for_selling', null);
            $price_group_id = request()->input('price_group', '');
            $product_types = request()->get('product_types', []);

            $search_fields = request()->get('search_fields', ['name', 'sku']);
            if (in_array('sku', $search_fields)) {
                $search_fields[] = 'sub_sku';
                 //$search_fields[] = 'product_description';
                
            }
          //  dd($search_fields);

            $result =$this->filterProduct($business_id, $search_term, $location_id, $not_for_selling, $price_group_id, $product_types, $search_fields, $check_qty,$hide_pos);

            return json_encode($result);
        }
    }
        public function filterProduct($business_id, $search_term, $location_id = null, $not_for_selling = null, $price_group_id = null, $product_types = [], $search_fields = [], $check_qty = false,$hide_pos = false, $search_type = 'like'){

        $query = \App\Product::join('variations', 'products.id', '=', 'variations.product_id')
                ->active()
                ->whereNull('variations.deleted_at')
                ->leftjoin('units as U', 'products.unit_id', '=', 'U.id')
                ->leftjoin(
                    'variation_location_details AS VLD',
                    function ($join) use ($location_id) {
                        $join->on('variations.id', '=', 'VLD.variation_id');

                        //Include Location
                        if (!empty($location_id)) {
                            $join->where(function ($query) use ($location_id) {
                                $query->where('VLD.location_id', '=', $location_id);
                                //Check null to show products even if no quantity is available in a location.
                                //TODO: Maybe add a settings to show product not available at a location or not.
                                $query->orWhereNull('VLD.location_id');
                            });
                            ;
                        }
                    }
                );

        if (!is_null($not_for_selling)) {
            $query->where('products.not_for_selling', $not_for_selling);
        }

        if (!empty($price_group_id)) {
            $query->leftjoin(
                'variation_group_prices AS VGP',
                function ($join) use ($price_group_id) {
                    $join->on('variations.id', '=', 'VGP.variation_id')
                        ->where('VGP.price_group_id', '=', $price_group_id);
                }
            );
        }

        $query->where('products.business_id', $business_id)
                ->where('products.type', '!=', 'modifier');

        if (!empty($product_types)) {
            $query->whereIn('products.type', $product_types);
        }

        if (in_array('lot', $search_fields)) {
            $query->leftjoin('purchase_lines as pl', 'variations.id', '=', 'pl.variation_id');
        }
        if($hide_pos=="true"){
           // return "test";
             $query->whereNull('products.hide_pos');
        }

        //Include search
        if (!empty($search_term)) {

            //Search with like condition
            if($search_type == 'like'){
                $query->where(function ($query) use ($search_term, $search_fields) {

                    if (in_array('name', $search_fields)) {
                        $query->where('products.name', 'like', '%' . $search_term .'%');
                    }
                    
                    if (in_array('sku', $search_fields)) {
                        $query->orWhere('sku', 'like', '%' . $search_term .'%');
                    }

                    if (in_array('sub_sku', $search_fields)) {
                        $query->orWhere('sub_sku', 'like', '%' . $search_term .'%');
                    }

                    if (in_array('lot', $search_fields)) {
                        $query->orWhere('pl.lot_number', 'like', '%' . $search_term .'%');
                    }
                });
            }

            //Search with exact condition
            if($search_type == 'exact'){
                $query->where(function ($query) use ($search_term, $search_fields) {

                    if (in_array('name', $search_fields)) {
                        $query->where('products.name', $search_term);
                    }
                    
                    if (in_array('sku', $search_fields)) {
                        $query->orWhere('sku', $search_term);
                    }

                    if (in_array('sub_sku', $search_fields)) {
                        $query->orWhere('sub_sku', $search_term);
                    }

                    if (in_array('lot', $search_fields)) {
                        $query->orWhere('pl.lot_number', $search_term);
                    }
                });
            }
        }


        //Include check for quantity
        if ($check_qty) {
            $query->where('VLD.qty_available', '>', 0);
        }

        if (!empty($location_id)) {
            $query->ForLocation($location_id);
        }

        $query->select(
                'products.id as product_id',
                'products.hide_pos',
                'products.name',
                'products.type',
                'variations.default_purchase_price',
                'products.enable_stock',
                'variations.id as variation_id',
                'variations.name as variation',
                'VLD.qty_available',
                'variations.sell_price_inc_tax as selling_price',
                'variations.sub_sku',
                'U.short_name as unit'
            );

        if (!empty($price_group_id)) {
            $query->addSelect('VGP.price_inc_tax as variation_group_price');
        }

        if (in_array('lot', $search_fields)) {
            $query->addSelect('pl.id as purchase_line_id', 'pl.lot_number');
        }

        $query->groupBy('variations.id');
        return $query->orderBy('VLD.qty_available', 'desc')
                        ->get();
    }

}